@@ -14,8 +14,11 @@ mailto: smstools@meinemullemaus.de
 #include <syslog.h>
 #include <stdio.h>
 #include <string.h>
+#include <signal.h>
 #include "blacklist.h"
 #include "extras.h"
+#include "logging.h"
+#include "alarm.h"
 
 int inblacklist(char* msisdn)
 {
@@ -50,7 +53,11 @@ int inblacklist(char* msisdn)
       fclose(file);
     }  
     else
-      syslog(LOG_ERR,"Cannot read blacklist file");
+    {
+      writelogfile(LOG_CRIT,"Cannot read blacklist file %s.",blacklist);
+      alarm_handler(LOG_CRIT,"SMSD","Cannot read blacklist file %s.",blacklist);
+      kill(0,SIGTERM);
+    }
   }      
   return 0;
 }
