@@ -16,6 +16,7 @@ mailto: smstools@meinemullemaus.de
 #include <fcntl.h>
 #include <termios.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <syslog.h>
@@ -26,6 +27,7 @@ mailto: smstools@meinemullemaus.de
 #include "modeminit.h"
 #include "logging.h"
 #include "alarm.h"
+#include "extras.h"
 
 char to[255] = {0};
 char subject[255] = {0};
@@ -36,7 +38,7 @@ char alarmhandler[256]={0};
 int alarmlevel=LOG_WARNING;
 int cs_convert;
 int keep;
-int is_binary=0;       // 1 if the sms is in binary format
+int alphabet=0;        // 0=GSM, 1=8bit Data, 2=UCS2, 3=reserved
 int with_udh=0;        // 1 if udh bit is set
 int is_statusreport=0; // 1 if the sms is a status report
 int quick=0;	       // 1 if quick mode
@@ -56,6 +58,7 @@ int octet2bin(char* octet) /* converts an octet to a 8-Bit value */
   return result;
 }
 
+
 int pdu2ascii(char* pdu, char* ascii) /* converts a PDU-String to Ascii */
 {                                     /* the first octet is the length */
   int bitposition=0;                  /* return the length of ascii */
@@ -67,8 +70,6 @@ int pdu2ascii(char* pdu, char* ascii) /* converts a PDU-String to Ascii */
   int octetcounter;
   char c;
   char binary[500];
-  int ext_characters=0;
-  int extended=0;
 
   /* First convert all octets to bytes */
   count=octet2bin(pdu);
@@ -88,31 +89,14 @@ int pdu2ascii(char* pdu, char* ascii) /* converts a PDU-String to Ascii */
       bitposition++;
       c=(c>>1)&127; /* The shift fills with 1, but I want 0 */
     }
-    if (cs_convert)
-    {
-      if (extended)
-      {
-        ascii[charcounter-ext_characters]=ext_convert(c,CS_SMS,CS_ISO);
-        extended=0;
-      }
-      else
-      {
-        ascii[charcounter-ext_characters]=convert(c,CS_SMS,CS_ISO);
-        // If this is an ESC character, then an extended character code follows
-        if (ascii[charcounter-ext_characters]==0x1B)
-        {
-          ext_characters++;
-          extended=1;
-        }
-      }
-    }
-    else if (c==0)
-      ascii[charcounter-ext_characters]=183;
+    // Replace ascii 0x00 by 0xB7 character because 0 is an invalid character in C
+    if (c=='\x00')
+      ascii[charcounter]='\xB7';
     else
-      ascii[charcounter-ext_characters]=c;
+      ascii[charcounter]=c;
   }
-  ascii[charcounter-ext_characters]=0;
-  return charcounter-ext_characters;
+  ascii[charcounter]=0;
+  return charcounter;
 }
 
 int pdu2binary(char* pdu, char* binary)
@@ -127,7 +111,7 @@ int pdu2binary(char* pdu, char* binary)
 }
 
 
-int getsms(int sim,char* pdu)  /* reads a SMS from the SIM-memory 1-10 */
+int getsms(int sim,char* pdu)  /* reads a SMS from the given SIM-memory */
 {                              /* returns number of SIM memory if successful */
 			       /* on digicom the return value can be != sim */
   char command[500];
@@ -138,12 +122,17 @@ int getsms(int sim,char* pdu)  /* reads a SMS from the SIM-memory 1-10 */
   char tmp[32];
   int i;
 
+  pdu[0]=0;
+
   if (strcmp(mode,"digicom")==0) // Digicom reports date+time only with AT+CMGL
   {
     writelogfile(LOG_INFO,"Trying to get stored message");
     sprintf(command,"AT+CMGL=\"ALL\"\r");
-    put_command(command,answer,sizeof(answer),200,0);
+    put_command(command,answer,sizeof(answer),200,"(\\+CMGL:.*OK)|(ERROR)");
     /* search for beginning of the answer */
+    /* unfortunately gcc v. 2 is not sure that the for cycle below will
+       initalize this variable, so set it to 0 to get rid of the warning */
+    position=0;
     for (i=1; i<11; i++)
     {
       sprintf(tmp,"+CMGL: %i",i);
@@ -161,7 +150,7 @@ int getsms(int sim,char* pdu)  /* reads a SMS from the SIM-memory 1-10 */
   {
     writelogfile(LOG_INFO,"Trying to get stored message %i",sim);
     sprintf(command,"AT+CMGR=%i\r",sim);
-    put_command(command,answer,sizeof(answer),50,0);
+    put_command(command,answer,sizeof(answer),50,"(\\+CMGR:.*OK)|(ERROR)");
     /* search for beginning of the answer */
     position=strstr(answer,"+CMGR:");
   }
@@ -192,13 +181,12 @@ void deletesms(int sim) /* deletes the selected sms from the sim card */
   char answer[500];
   writelogfile(LOG_INFO,"Deleting message %i",sim);
   sprintf(command,"AT+CMGD=%i\r",sim);
-  put_command(command,answer,sizeof(answer),50,0);
+  put_command(command,answer,sizeof(answer),50,"(OK)|(ERROR)");
 }
 
 
 void check_memory(int *used_memory,int *max_memory) // checks the size of the SIM memory
 {
-  char command[100];
   char answer[500];
   char* posi;
   int laenge;
@@ -207,19 +195,19 @@ void check_memory(int *used_memory,int *max_memory) // checks the size of the SI
   *max_memory=10;
 
   writelogfile(LOG_INFO,"Checking memory size");
-  put_command("AT+CPMS?\r",answer,sizeof(answer),50,0);
-  if (posi=strstr(answer,"+CPMS:"))
+  put_command("AT+CPMS?\r",answer,sizeof(answer),50,"(\\+CMPS:.*OK)|(ERROR)");
+  if ((posi=strstr(answer,"+CPMS:")))
   {
     // Modem supports CPMS command. Read memory size
-    if (posi=strchr(posi,','))
+    if ((posi=strchr(posi,',')))
     {
       posi++;
-      if (laenge=strcspn(posi,",\r"))
+      if ((laenge=strcspn(posi,",\r")))
       {
         posi[laenge]=0;
         *used_memory=atoi(posi);
         posi+=laenge+1;
-	if (laenge=strcspn(posi,",\r"))
+	if ((laenge=strcspn(posi,",\r")))
 	{
           *max_memory=atoi(posi);
   	  writelogfile(LOG_INFO,"Used memory is %i of %i",*used_memory,*max_memory);
@@ -282,12 +270,13 @@ int splitascii(char* source, char* sendr, char* name, char* date, char* time, ch
 
 
 // Subroutine for splitpdu() for messages type 0 (SMS-Deliver)
-// Returns the length of the ascii string. In binary mode ascii contains the binary SMS
-int split_type_0(char* Pointer,char* sendr,char* date,char* time,char* ascii)
+// Returns the length of the received message.
+int split_type_0(char* Pointer,char* sendr,char* date,char* time,char* result)
 {
   int Length;
   int padding;
   char tmpsender[100];
+  char ascii[200];
   Length=octet2bin(Pointer);
   padding=Length%2;
   Pointer+=2;
@@ -309,19 +298,22 @@ int split_type_0(char* Pointer,char* sendr,char* date,char* time,char* ascii)
       sendr[Length-1]=0;
   }
   Pointer=Pointer+Length+padding+3;
-  if ((Pointer[0] & 4)==4)
-    is_binary=1;
-  else
-    is_binary=0;
+  alphabet=(Pointer[0] & 12) >>2;
   Pointer++;
   sprintf(date,"%c%c-%c%c-%c%c",Pointer[1],Pointer[0],Pointer[3],Pointer[2],Pointer[5],Pointer[4]);
   Pointer=Pointer+6;
   sprintf(time,"%c%c:%c%c:%c%c",Pointer[1],Pointer[0],Pointer[3],Pointer[2],Pointer[5],Pointer[4]);
   Pointer=Pointer+8;
-  if (is_binary)
-    return pdu2binary(Pointer,ascii);
-  else
-    return pdu2ascii(Pointer,ascii);
+  if ((alphabet==0) && (cs_convert))
+  {
+    Length=pdu2ascii(Pointer,ascii);
+    Length=gsm2iso(ascii,result,200);
+    return Length;
+  }
+  else if (alphabet==0)
+    return pdu2ascii(Pointer,result);
+  else  
+    return pdu2binary(Pointer,result);
 }
 
 
@@ -417,7 +409,6 @@ int split_type_2(char* position,char* from, char* date,char* time,char* result)
 int splitpdu(char* pdu, char* sendr, char* from, char* name, char* date, char* time, char* ascii, char* smsc)
 {
   int Length;
-  int padding;
   int Type;
   char* Pointer;
   char* start;
@@ -445,7 +436,7 @@ int splitpdu(char* pdu, char* sendr, char* from, char* name, char* date, char* t
   /* the pdu is after the first \r */
   start=strstr(end+1,"\r");
   if (start==0)
-    return;
+    return 0;
   pdu=++start;
   /* removes unwanted ctrl chars at the beginning */
   while (pdu[0] && (pdu[0]<=' '))
@@ -495,11 +486,12 @@ void help()
   printf("              -bx  set baudrate to x (default %i)\n",baudrate);
   printf("              -c   use character set conversion\n");
   printf("              -dx  set modem device to x (default %s)\n",device);
+  printf("              -Dx  delay sending characters by x milliseconds (default %d\n",send_delay);
   printf("              -ex  wait x seconds before retry (default %d)\n",errorsleeptime);
   printf("              -h   this help\n");
   printf("              -H   disable hardware handshake RTS/CTS\n");
   printf("              -ix  modem init string x\n");
-  printf("              -k   keep all SMS in the SIM memory\n",device);
+  printf("              -k   keep all SMS in the SIM memory\n");
   printf("              -lx  use logfile x (filename or handle) (default syslog)\n");
   printf("              -Lx  use loglevel x (default %i)\n",loglevel);
   printf("              -mx  set PDU mode to x (default %s)\n",mode);
@@ -507,20 +499,17 @@ void help()
   printf("              -nx  set modem name to x\n");
   printf("              -ox  file result to write (default stdout)\n");
   printf("              -px  set pin to x (only needed if modem not initialized)\n");
-  printf("	        -q   quick initialization\n");
-  printf("              -Q   quicker (check only the first SIM memory)\n");
+  printf("              -q   quick, no modem initialization\n");
+  printf("              -Q   quicker, check only the first SIM memory\n");
   printf("              -sx  generate a Subject: line with value x\n");
   printf("              -tx  generate a To: line with value x\n");
   printf("              -V   print version info and copyright\n\n");
-  printf("All options may be omitted. Output is written do stdout.\n");
   printf("After getting the SMS it will be deleted from the SIM card.\n\n");
-  printf("Example:\n");
-  printf("              getsms -d/dev/ttyS0 -p1234 -s\"This is a test\"\n");
   printf("Return codes:\n");
   printf("              0   received one SMS\n");
   printf("              1   cannot open device\n");
-  printf("              2   cannot verify PIN or modem not ready.\n");
-  printf("              3   modem not registered to the network\n");
+  printf("              2   PIN is wrong\n");
+  printf("              3   modem initialisation problem\n");
   printf("              4   no SMS available\n");
   printf("              5   received SMS Status Report\n");
   exit(0);
@@ -535,17 +524,18 @@ void parsearguments(int argc,char** argv)
   pin[0]=0;
   to[0]=0;
   baudrate=19200;
+  send_delay=1;
   cs_convert=0;
   keep=0;
   strcpy(subject,"SMS from GSM modem");
   initstring[0]=0;
+  initstring2[0]=0;
   errorsleeptime=10;
   modemname[0]=0;
   rtscts=1;
-  quickinit=0;
   do
   {
-    result=getopt(argc,argv,"a:A:n:b:e:o:l:L:khHqQi:cd:t:s:m:p:V");
+    result=getopt(argc,argv,"a:A:n:b:e:o:l:L:khHqQi:j:cd:D:t:s:m:p:V");
     switch (result)
     {
       case 'h': help();
@@ -560,9 +550,9 @@ void parsearguments(int argc,char** argv)
                 break;
       case 'c': cs_convert=1;
       		break;
-      case 'q': quickinit=1;
+      case 'q': quick=1;
                 break;
-      case 'Q': quick=1;
+      case 'Q': quick=2;
       		break;
       case 'd': strcpy(device,optarg);
                 break;
@@ -570,9 +560,14 @@ void parsearguments(int argc,char** argv)
       		break;
       case 'b': baudrate=atoi(optarg);
                 break;
+      case 'D': send_delay=atoi(optarg);
+                break;
       case 'i': strcpy(initstring,optarg);
       		strcat(initstring,"\r");
       		break;
+      case 'j': strcpy(initstring2,optarg);
+                strcat(initstring2,"\r");
+                break;
       case 'l': strcpy(logfile,optarg);
                 break;
       case 'L': loglevel=atoi(optarg);
@@ -628,7 +623,7 @@ int main(int argc,char** argv)
   int userdatalength;
   int sim;
   char pdu[1024];
-  char ascii[1024];
+  char ascii[200];
   char sendr[100];
   char smsc[31];
   char name[64];
@@ -646,10 +641,16 @@ int main(int argc,char** argv)
   writelogfile(LOG_INFO,"Checking incoming SMS");
   openmodem();
   setmodemparams();
-  initmodem();
-  if (!quick)
+  if (quick==0)  // full initialisation and memory check
+  {
+    initmodem();
     check_memory(&used_memory,&max_memory);
-  else
+  }
+  else if (quick==1)  // only memory check
+  {
+    check_memory(&used_memory,&max_memory);
+  }
+  else  // no memory check, assume that there is 1 message to read
   {
     used_memory=1;
     max_memory=1;
@@ -659,7 +660,7 @@ int main(int argc,char** argv)
   {
     for (sim=1; sim<=max_memory; sim++)
     {
-      if (found=getsms(sim,pdu))
+      if ((found=getsms(sim,pdu)))
       {
         // Ok, now we split the PDU string into parts and show it
         if ((strcmp(mode,"ascii")==0) || (strcmp(mode,"digicom")==0))
@@ -669,10 +670,10 @@ int main(int argc,char** argv)
 
 	writelogfile(LOG_NOTICE, "SMS received, From: %s",sendr);
 	alarm_handler(LOG_NOTICE,"", "SMS received, From: %s",sendr);
-        if (is_binary==0)
+	if (alphabet==0)
           writelogfile(LOG_DEBUG,"Message: %s", ascii);
         else
-          writelogfile(LOG_DEBUG,"Message has binary data\n\n",userdatalength);
+          writelogfile(LOG_DEBUG,"Message has %i bytes",userdatalength);
 
         if (outputfile[0])
 	  fd = fopen(outputfile, "w");
@@ -699,10 +700,17 @@ int main(int argc,char** argv)
           }
 	  if (subject[0])
 	    fprintf(fd, "Subject: %s\n",subject);
-          if (is_binary==0)
-  	    fprintf(fd, "\n%s\n",ascii);
-	  else
+          if (alphabet==0)
 	  {
+	    if (cs_convert)
+	      fprintf(fd,"Alphabet: ISO8859-15\n\n");
+	    else
+	      fprintf(fd,"Alphabet: GSM\n\n");
+  	    fprintf(fd, "%s",ascii);
+	  }
+	  else if (alphabet==1)
+	  {
+	    fprintf(fd,"Alphabet: binary\n");
 	    if (with_udh)
 	      fprintf(fd,"UDH: true\n");
 	    else
@@ -711,7 +719,23 @@ int main(int argc,char** argv)
 	    if (outputfile[0])
 	      fwrite(ascii,1,userdatalength,fd);
 	    else
-	      fprintf(fd, "binary not displayed\n");
+	      fprintf(fd, "binary data not displayed\n");
+	  }
+	  else if (alphabet==2)
+	  {
+	    fprintf(fd,"Alphabet: UCS2\n\n");
+	    if (outputfile[0])
+	      fwrite(ascii,1,userdatalength,fd);
+	    else
+	      fprintf(fd, "UCS2 text not displayed\n");
+	  }
+	  else if (alphabet==3)
+	  {
+	    fprintf(fd,"Alphabet: reserved\n\n");
+	    if (outputfile[0])
+	      fwrite(ascii,1,userdatalength,fd);
+	    else
+	      fprintf(fd, "data with unknown alphabet not displayed\n");
 	  }
 	  if (outputfile[0]) 
             fclose(fd);
