@@ -14,9 +14,11 @@ mailto: smstools@meinemullemaus.de
 #include "locking.h"
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
 #include <string.h>
 #include <fcntl.h>
 #include <limits.h>
+#include <unistd.h>
 
 int lockfile(const char* const filename)
 {
@@ -58,7 +60,6 @@ int islocked(const char* const filename)
 int unlock(const char* const filename)
 {
   char lockfilename[PATH_MAX];
-  struct stat statbuf;
   strcpy(lockfilename,filename);
   strcat(lockfilename,".LOCK");
   if (unlink(lockfilename))
