@@ -20,9 +20,10 @@ int modem; // File handle of serial port
 int baudrate;
 int errorsleeptime;  // Time to wait after ERROR
 int rtscts; // 1 enables hardware handshake via RTS/CTS
-int quickinit; // 1 makes a quicker initialization.
+int send_delay;  // Makes sending characters to the modem slower
 char device[100]; // Name of serial port
 char initstring[100]; // Initialisation string
+char initstring2[100]; // Initialisation string
 char modemname[100]; // Name of modem
 char smsc[100]; // Number of SMSC
 char pin[16]; // PIN for the SIM card
@@ -32,9 +33,9 @@ struct termios oldtio;
 // put_command
 // Sends a command to the modem and waits max timout*0.1 seconds for an answer.
 // The function returns the length of the answer.
-// The answer can be Ok, ERROR or expect. After getting the answer the
-// functions reads 0.1s more and returns then.
-// The command may be empty or NULL. 
+// The function waits until a timeout occurs or the expected answer occurs. 
+// The command and expect may be empty or NULL. 
+// expect is an extended regular expression.
 
 int put_command(char* command,char* answer,int max,int timeout,char* expect);
 
