@@ -16,6 +16,7 @@ mailto: smstools@meinemullemaus.de
 #include <fcntl.h>
 #include <termios.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #include <syslog.h>
@@ -24,12 +25,14 @@ mailto: smstools@meinemullemaus.de
 #include "modeminit.h"
 #include "logging.h"
 #include "alarm.h"
+#include "extras.h"
 #include <limits.h>
+#include <errno.h>
 
 char message[500]={0};
-int is_binary;  // 1 is binary file
-int udh=1;       // disable UDH bit
-int messagelen; // length of message
+int alphabet=0;  // 0=GSM, 1=8bit Data, 2=UCS2, 3=reserved
+int udh=1;       // enable UDH bit
+int messagelen=0; // length of message
 char filename[PATH_MAX]={0};
 char logfile[PATH_MAX]={0};
 char alarmhandler[PATH_MAX]={0};
@@ -39,22 +42,25 @@ char to[50]={0};
 int cs_convert=0;
 int report=0;
 int flash_sms=0;
+int quick=0;
 char idfile[PATH_MAX]={0};
 
 void help()
 {
   printf("putsms sends an SMS through a GSM 07.05 compatible modem.\n");
   printf("Usage:\n");
-  printf("              putsms [options] destination \"message\"\n");
+  printf("              putsms [options] destination [\"message\"]\n");
   printf("Options:\n");
   printf("              -ax  set alarm handler to x (default none)\n");
   printf("              -Ax  set alarm level to x (default %i)\n",alarmlevel);
   printf("              -bx  set baudrate to x (default %i)\n",baudrate);
   printf("              -c   use character set conversion\n");
   printf("              -dx  set modem device to x (default %s)\n",device);
+  printf("              -Dx  delay sending characters by x milliseconds (default %d)\n",send_delay);
   printf("              -ex  wait x seconds before retry (default %d)\n",errorsleeptime);
   printf("              -fx  message file x (ascii)\n");
-  printf("              -Fx  message file x (binary, only in pdu mode old or new)\n");
+  printf("              -Fx  message file x (binary, only in pdu mode)\n");
+  printf("              -gx  message file x (UCS2, only in pdu mode)\n");
   printf("              -h   this help\n");
   printf("              -H   disable hardware handshake RTS/CTS\n");
   printf("              -ix  modem init string x\n");
@@ -64,32 +70,21 @@ void help()
   printf("              -mx  set the PDU mode to x (default %s)\n",mode);
   printf("                   x can be old, new, ascii or digicom\n");
   printf("              -nx  set modem name to x\n");
-  printf("              -px  set pin to x (default %s. Only needed if\n",pin);
-  printf("                   modem not initialized)\n");
-  printf("              -q   quick initialisation\n");
+  printf("              -px  set pin to x (only needed if modem not initialized)\n");
+  printf("              -q   quick, no modem initialisation\n");
   printf("              -r   request status report SMS (not in ascii mode)\n");
   printf("              -Rx  report message ID in file x\n");
   printf("              -sx  set SMSC Number to x\n");
   printf("              -u   no user data header (UDH bit). Only used with -F.\n");
   printf("              -V   print version info and copyright\n\n");
-  printf("All options may be omitted.\n\n");
-  printf("If you've specified a message file you may omitt the message argument.\n");
-  printf("If the message file has a To: header you may omitt the destination argument.\n\n");
-  printf("The destination number and SMSC have to be the format like below.\n");
-  printf("Example:\n");
-  printf("              putsms -d/dev/ttyS0 491721234567 \"Hello\"\n");
   printf("Return codes:\n");
   printf("              0   success\n");
   printf("              1   cannot open device or file\n");
-  printf("              2   cannot verify PIN or modem not ready\n");
-  printf("              3   modem not registered to the network or did not accept PDU mode.\n");
+  printf("              2   PIN is wrong\n");
+  printf("              3   problem with modem initialisation\n");
   printf("              4   SMS could not be sent\n");
   printf("              5   error in arguments\n\n");
-  printf("Format of the message file: \n");
-  printf("To: xxxxxxxx<NL>other headers<NL><NL>message\n");
-  printf("putsms ignores the other headers. The message begins after an empty\n");
-  printf("line. You may specify a message file that contains only the message\n");
-  printf("without header beginning at the first line.\n\n");
+  printf("The message file can contain a header but this is not used.\n");
   exit(0);
 }
 
@@ -113,9 +108,6 @@ void loadmessagefile()
   int File;
   int readcount;
   char* position;
-  char* endpos;
-  int Length;
-  char tmp[500];
 
   /* load Message-File (if set) */
   if (filename[0])
@@ -124,7 +116,8 @@ void loadmessagefile()
     File = open(filename, O_RDONLY);
     if (File<0)
     {
-      perror(device);
+      writelogfile(LOG_ERR,"Could not open file %s, cause: %s",filename,strerror(errno));
+      alarm_handler(LOG_ERR,"","Could not open file %s, cause: %s",filename,strerror(errno));
       exit(1);
     }
     else
@@ -132,45 +125,17 @@ void loadmessagefile()
       readcount=read(File,message,sizeof(message)-1);
       message[readcount]=0;
       messagelen=readcount;
-      /* Search for To: */
-      position=strstr(message,"To: ");
-      if (position!=message)
-      {
-        position=strstr(message,"\nTo: ");
-	if (position)
-	  position++;
-      }
-      if (position)
-      {
-        /* found, get the line */
-        endpos=strchr(position+4,'\r');
-	if (!endpos)
-	  endpos=strchr(position+4,'\n');
-	if (endpos)
-	{
-	  Length=endpos-position-4;
-	  strncpy(to,position+4,Length);
-	  to[Length]=0;
-	}
-
-	/* Get the rest of the message, skip newline-characters */
-	position=strstr(message,"\n\n");
-	if (position)
+      /* skip newline-characters */
+	if ((position=strstr(message,"\n\n")))
 	  position+=2;
-	else
-	{
-	  position=strstr(message,"\r\n\r\n"); /* for windows compatibility */
-	  if (position)
-	    position+=4;
-	}
-
+	else if ((position=strstr(message,"\r\n\r\n")))
+	  position+=4;
 	if (position) /* if the double newline was found, cut the whole header out */
 	{
 	  messagelen=messagelen-(position-message);
 	  memmove(message,position,messagelen);
 	  message[messagelen]=0;
 	}
-      }
       close(File);
     }
   }
@@ -181,7 +146,6 @@ void loadmessagefile()
 void parsearguments(int argc,char** argv)
 {
   int result;
-  char tmp[500];
 
   /* set default values */
   strcpy(device,"/dev/ttyS0");
@@ -190,20 +154,22 @@ void parsearguments(int argc,char** argv)
   filename[0]=0;
   smsc[0]=0;
   baudrate=19200;
-  is_binary=0;
+  send_delay=1;
+  alphabet=0;
   initstring[0]=0;
+  initstring2[0]=0;
   report=0;
   errorsleeptime=10;
   modemname[0]=0;
   smsc[0]=0;
   rtscts=1;
-  quickinit=0;
+  quick=0;
   flash_sms=0;
   idfile[0]=0;
   /* parse arguments */
   do
   {
-    result=getopt(argc,argv,"a:A:n:b:l:L:ce:rR:qhHIi:d:p:m:s:f:F:Vu");
+    result=getopt(argc,argv,"a:A:n:b:l:L:ce:rR:qhHIi:j:d:D:p:m:s:f:F:g:Vu");
     switch (result)
     {
       case 'h': help();
@@ -214,6 +180,8 @@ void parsearguments(int argc,char** argv)
       		break;
       case 'b': baudrate=atoi(optarg);
                 break;
+      case 'D': send_delay=atoi(optarg);
+                break;
       case 'c': cs_convert=1;
                 break;
       case 'I': flash_sms=1;
@@ -224,7 +192,7 @@ void parsearguments(int argc,char** argv)
     		break;
       case 'R': strcpy(idfile,optarg);
                 break;
-      case 'q': quickinit=1;
+      case 'q': quick=1;
                 break;
       case 'H': rtscts=0;
 		break;
@@ -245,13 +213,19 @@ void parsearguments(int argc,char** argv)
       case 's': strcpy(smsc,optarg);
                 break;
       case 'f': strcpy(filename,optarg);
-    		is_binary=0;
+    		alphabet=0;
                 break;
       case 'i': strcpy(initstring,optarg);
       	        strcat(initstring,"\r");
 		break;
+      case 'j': strcpy(initstring2,optarg);
+                strcat(initstring2,"\r");
+                break;
       case 'F': strcpy(filename,optarg);
-    		is_binary=1;
+    		alphabet=1;
+		break;
+      case 'g': strcpy(filename,optarg);
+		alphabet=2;
 		break;
       case 'V': printf("Version %s, Copyright (c) by Stefan Frings, smstools@meinemullemaus.de\n",putsms_version);
                 exit(0);
@@ -313,67 +287,45 @@ void check_arguments(int argc,char** argv)
     exit(5);
   }
   /* Check if binary file allowed */
-  if (is_binary && (strcmp(mode,"ascii")==0))
+  if (alphabet!=0 && (strcmp(mode,"ascii")==0))
   {
-    writelogfile(LOG_ERR,"Binary files are not allowed in ascii mode");
+    writelogfile(LOG_ERR,"Binary and UCS2 files are not allowed in ascii mode");
     exit(5);
   }
 
 }
 
-/* Work with the complex bit building to generate a 7 bit PDU string encapsulated in 8 bit. Returns number of 
-7 Bit charcters */
+/* Work with the complex bit building to generate a 7 bit PDU string encapsulated in 8 bit. Returns number of 7 Bit charcters */
+
 int ascii2pdu(char* ascii,char* pdu)
 {
   char tmp[500];
   char octett[10];
   int pdubitposition;
-  int pdubyteposition;
+  int pdubyteposition=0;
   int asciiLength;
   int character;
   int bit;
   int pdubitnr;
   char converted;
-  int ext_characters=0;
   int counted_characters=0;
   asciiLength=strlen(ascii);
+  if (asciiLength>160)
+    asciiLength=160;
+  //clear the tmp buffer
   for (character=0;character<sizeof(tmp);character++)
     tmp[character]=0;
-  for (character=0;((character+ext_characters)<160) && (character<asciiLength);character++)
+  for (character=0;character<asciiLength;character++)
   {
-    if (cs_convert)
-    {
-      // Is the character an extended character?
-      converted=ext_convert(ascii[character],CS_ISO,CS_SMS);
-      if (converted!=' ')
-      {
-        counted_characters++;
-         // It is an extended character. Insert ESC first.
-         for (bit=0;bit<7;bit++)
-         {
-           pdubitnr=7*(character+ext_characters)+bit;
-           pdubyteposition=pdubitnr/8;
-           pdubitposition=pdubitnr%8;
-           if (0x1B & (1<<bit))
-             tmp[pdubyteposition]=tmp[pdubyteposition] | (1<<pdubitposition);
-           else
-             tmp[pdubyteposition]=tmp[pdubyteposition] & ~(1<<pdubitposition);
-         }
-         ext_characters++;
-       }
-       else
-         // Is is a regular character
-         converted=convert(ascii[character],CS_ISO,CS_SMS);
-    }
-    else if (ascii[character]==-73)
-      converted=0;
+    // replace 0xB7 by 0x00, because this is a replacement for 0x00 that is the @ character is GSM alphabet 
+    if (ascii[character]=='\xB7')
+      converted='\x00';
     else
       converted=ascii[character];
-
     counted_characters++;
     for (bit=0;bit<7;bit++)
     {
-      pdubitnr=7*(character+ext_characters)+bit;
+      pdubitnr=7*character+bit;
       pdubyteposition=pdubitnr/8;
       pdubitposition=pdubitnr%8;
       if (converted & (1<<bit))
@@ -381,7 +333,6 @@ int ascii2pdu(char* ascii,char* pdu)
       else
         tmp[pdubyteposition]=tmp[pdubyteposition] & ~(1<<pdubitposition);
     }
-
   }
   tmp[pdubyteposition+1]=0;
   pdu[0]=0;
@@ -415,6 +366,7 @@ void make_pdu(char* nummer, char* message, int messagelen, char* pdu)
   int flags;
   char tmp[50];
   char tmp2[500];
+  char tmp3[500];
   int numberformat;
   int numberlength;
 
@@ -435,32 +387,46 @@ void make_pdu(char* nummer, char* message, int messagelen, char* pdu)
   // Swap every second character
   swapchars(tmp);
   flags=1; // SMS-Sumbit MS to SMSC
-  if (flash_sms)
-     coding=240+0; /* Dummy + Class 0 */
-  else
+  
+  if (alphabet==1)
   {
-    coding=240+1; // Dummy + Class 1
-    if (is_binary)
-    {
-      coding+=4; // 8 Bit
-      if (udh)
-        flags+=64; // User Data Header
-    }
+    coding=244; // 8 bit binary
+    if (udh)
+      flags+=64; // User Data Header
   }
+  else if (alphabet==2)
+    coding=24; // UCS2
+  else
+    coding=240; // 7bit text
+
+  if (flash_sms)
+    coding+=0; // Class 0
+  else
+    coding+=1; // Class1
+
   if (strcmp(mode,"old")!=0)
     flags+=16; // Validity field
   if (report)
     flags+=32; /* Request Status Report */
   /* Create the PDU string of the message */
-  if (is_binary)
+  if (alphabet==1 || alphabet==2)
     binary2pdu(message,messagelen,tmp2);
   else
-    messagelen=ascii2pdu(message,tmp2);
+  {
+    // Convert character set if source=ISO and conversion enabled
+    if ((alphabet==0) && (cs_convert))
+    {
+      messagelen=iso2gsm(message,tmp3,sizeof(tmp3));
+      messagelen=ascii2pdu(tmp3,tmp2);
+    }
+    else
+      messagelen=ascii2pdu(message,tmp2);
+  }
   /* concatenate the first part of the PDU string */
   if (strcmp(mode,"old")==0)
     sprintf(pdu,"%02X00%02X%02X%s00%02X%02X",flags,numberlength,numberformat,tmp,coding,messagelen);
   else
-    sprintf(pdu,"00%02X00%02X%02X%s00%02XA7%02X",flags,numberlength,numberformat,tmp,coding,messagelen);
+   sprintf(pdu,"00%02X00%02X%02X%s00%02XA7%02X",flags,numberlength,numberformat,tmp,coding,messagelen);
   /* concatenate the text to the PDU string */
   strcat(pdu,tmp2);
 }
@@ -493,9 +459,9 @@ void putsms()
   while (1)
   {
     retries+=1;
-    put_command(command,answer,sizeof(answer),50,">");
+    put_command(command,answer,sizeof(answer),50,"(\\>)|(ERROR)");
     if (! strstr(answer,"ERROR"))
-      put_command(command2,answer,sizeof(answer),300,0);
+      put_command(command2,answer,sizeof(answer),300,"(OK)|(ERROR)");
     if (strstr(answer,"ERROR"))
     {
       writelogfile(LOG_ERR,"Uups, the modem said ERROR.");
@@ -505,9 +471,8 @@ void putsms()
       {
         writelogfile(LOG_NOTICE,"Waiting %i sec. before retrying",errorsleeptime);
 	sleep(errorsleeptime);
-	// the next line is a workaround for an unknown buggy gsm modem.
-	put_command("\r\x1A\r",answer,sizeof(answer),10,0);
-	sleep(1);
+	// Force a new initialisation after an ERROR
+	initmodem();
       }
       else
         exit(4);
@@ -553,7 +518,8 @@ int main(int argc,char** argv)
   writelogfile(LOG_INFO,"Sending SMS");
   openmodem();
   setmodemparams();
-  initmodem();
+  if (quick==0)
+    initmodem();
   putsms();
   tcsetattr(modem,TCSANOW,&oldtio);
   return 0;
