@@ -12,9 +12,10 @@ mailto: smstools@meinemullemaus.de
 */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include "syslog.h"
+#include <syslog.h>
 #include "extras.h"
 #include "cfgfile.h"
 #include "smsd_cfg.h"
@@ -41,7 +42,7 @@ void initcfg()
   checkhandler[0]=0;
   alarmhandler[0]=0;
   logfile[0]=0;
-  loglevel=LOG_WARNING;
+  loglevel=-1;  // Will be changed after reading the cfg file if stil -1
   alarmlevel=LOG_WARNING;
   
   strcpy(d_spool,"/var/spool/sms/outgoing");
@@ -68,13 +69,16 @@ void initcfg()
     devices[i].pin[0]=0;
     devices[i].smsc[0]=0;
     devices[i].baudrate=19200;
-    devices[i].cs_convert=0;
+    devices[i].send_delay=1;
+    devices[i].cs_convert=1;
     devices[i].initstring[0]=0;
+    devices[i].initstring2[0]=0;
     devices[i].eventhandler[0]=0;
     devices[i].report=0;
-    devices[i].rtscts=0;
+    devices[i].rtscts=1;
     strcpy(devices[i].mode,"new");
-    for (j=0; j<PROVIDER; j++)
+    strcpy(devices[i].queues[0],d_checked);
+    for (j=1; j<PROVIDER; j++)
       devices[i].queues[j][0]=0;
   }
 }
@@ -150,8 +154,6 @@ void readcfg()
     }
     if (result==-1)
       fprintf(stderr,"Syntax Error in config file: %s\n",value);
-    for (i=0; i<DEVICES; i++)
-      strcpy(devices[i].queues[0],d_checked);
     /* read queue-settings */
     if (!gotosection(File,"queues"))
       fprintf(stderr,"Info: Could not find [queues] in config file.\n");
@@ -233,8 +235,12 @@ void readcfg()
 	    strcpy(devices[i].smsc,value);
 	  else if (strcmp(name,"baudrate")==0)
 	    devices[i].baudrate=atoi(value);
+          else if (strcmp(name,"send_delay")==0)
+            devices[i].send_delay=atoi(value);
 	  else if (strcmp(name,"init")==0)
 	    strcpy(devices[i].initstring,value);
+          else if (strcmp(name,"init2")==0)
+            strcpy(devices[i].initstring2,value);
 	  else if (strcmp(name,"eventhandler")==0)
     	    strcpy(devices[i].eventhandler,value);
 	  else if (strcmp(name,"report")==0)
@@ -242,17 +248,27 @@ void readcfg()
 	  else if (strcmp(name,"rtscts")==0)
     	    devices[i].rtscts=yesno(value);
 	  else
-	    fprintf(stderr,"Unknown variable in config file [%s]: %s\n",device_name,name);
+	    fprintf(stderr,"Unknown variable in config file for modem %s: %s\n",device_name,name);
 	  result=getline(File,name,sizeof(name),value,sizeof(value));
 	}
         if (result==-1)
           fprintf(stderr,"Syntax Error in config file: %s\n",value);
+        if ((devices[i].report==1) && (devices[i].incoming==0))
+          fprintf(stderr,"Warning: Cannot receive status reports because receiving is disabled on modem %s\n",device_name);
       }
       else
         break;
     }
     fclose(File);
     set_alarmhandler(alarmhandler,alarmlevel,"SMSD");
+    // if loglevel is unset, then set it depending on if we use syslog or a logfile
+    if (loglevel==-1)
+    {
+      if (logfile[0]==0)
+        loglevel=LOG_DEBUG;
+      else
+        loglevel=LOG_WARNING;
+    }
   }
   else
     fprintf(stderr,"Cannot open config file for read.\n");
