@@ -12,14 +12,22 @@ mailto: smstools@meinemullemaus.de
 */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <limits.h>
+#include "alarm.h"
 #include "smsd_cfg.h"
 #include "stats.h"
 #include <syslog.h>
 #include "logging.h"
 #include <errno.h>
+#ifndef NOSTATS
+#include <mm.h>
+#endif
+
+char newstatus[DEVICES+1]={0};
+char oldstatus[DEVICES+1]={0};
 
 void initstats()
 {
@@ -33,9 +41,9 @@ void initstats()
   for (i=0; i<DEVICES; i++)
   {
 #ifndef NOSTATS
-    if (statistics[i]=(_stats*)MM_malloc(sizeof(_stats)))
+    if ((statistics[i]=(_stats*)MM_malloc(sizeof(_stats))))
 #else
-    if (statistics[i]=(_stats*)malloc(sizeof(_stats)))
+    if ((statistics[i]=(_stats*)malloc(sizeof(_stats))))
 #endif
     {
       statistics[i]->succeeded_counter=0;
@@ -129,18 +137,23 @@ void loadstats()
   }    
 }
 
-void print_status()
+void print_status(char* buffer)
 {
   int j;
   if (printstatus)
   {
+    strcpy(oldstatus,newstatus);
     for (j=0; j<DEVICES; j++)
-      write(1,&statistics[j]->status,1);
-    write(1,&"\n",1);
-    fflush(stdout);
+      newstatus[j]=statistics[j]->status;
+    newstatus[DEVICES]=0;
+    if (strcmp(oldstatus,newstatus))
+    {
+      printf("%s\n",newstatus);
+    }
   }
-}
-
+}  
+      
+      
 void checkwritestats()
 {
   time_t now,next_time;
@@ -165,7 +178,7 @@ void checkwritestats()
       if (datei)
       {
         fprintf(datei,"runtime,rejected\n");
-	fprintf(datei,"%i,%i\n\n",now-start_time,rejected_counter);
+	fprintf(datei,"%li,%i\n\n",now-start_time,rejected_counter);
 	fprintf(datei,"name,succeeded,failed,received,multiple_failed,usage_s,usage_r\n");
 	for (i=0; i<DEVICES; i++)
 	{
