@@ -14,8 +14,11 @@ mailto: smstools@meinemullemaus.de
 #include <syslog.h>
 #include <stdio.h>
 #include <string.h>
+#include <signal.h>
 #include "whitelist.h"
 #include "extras.h"
+#include "logging.h"
+#include "alarm.h"
 
 int inwhitelist(char* msisdn)
 {
@@ -40,17 +43,22 @@ int inwhitelist(char* msisdn)
 	    fclose(file);
             return 1;
 	  }  
-          else if (msisdn[0]=='s' && strncmp(msisdn,line,strlen(line))==0)
-          {
-            fclose(file);
-            return 1;
-          }
+	  else if (msisdn[0]=='s' && strncmp(msisdn+1,line,strlen(line))==0)
+	  {
+	    fclose(file);
+	    return 1;
+	  }
 	}  
       }  
       fclose(file);
+      return 0;
     }  
     else
-      syslog(LOG_ERR,"Cannot read whitelist file");
+    {
+      writelogfile(LOG_CRIT,"Cannot read whitelist file %s.",whitelist);
+      alarm_handler(LOG_CRIT,"SMSD","Cannot read whitelist file %s.",whitelist);
+      kill(0,SIGTERM);
+    }
   }      
-  return 0;
+  return 1;
 }
